/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was com.redhat.NewPrinterNotification.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#ifndef NEWPRINTERNOTIFICATIONADAPTOR_H
#define NEWPRINTERNOTIFICATIONADAPTOR_H

#include <QtCore/QObject>
#include <QtDBus/QtDBus>
#include "NewPrinterNotification.h"
#include <QtCore/qcontainerfwd.h>

/*
 * Adaptor class for interface com.redhat.NewPrinterNotification
 */
class NewPrinterNotificationAdaptor: public QDBusAbstractAdaptor
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "com.redhat.NewPrinterNotification")
    Q_CLASSINFO("D-Bus Introspection", ""
"  <interface name=\"com.redhat.NewPrinterNotification\">\n"
"    <method name=\"GetReady\"/>\n"
"    <method name=\"NewPrinter\">\n"
"      <arg direction=\"in\" type=\"i\" name=\"status\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"name\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"mfg\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"mdl\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"des\"/>\n"
"      <arg direction=\"in\" type=\"s\" name=\"cmd\"/>\n"
"    </method>\n"
"  </interface>\n"
        "")
public:
    NewPrinterNotificationAdaptor(NewPrinterNotification *parent);
    ~NewPrinterNotificationAdaptor() override;

    inline NewPrinterNotification *parent() const
    { return static_cast<NewPrinterNotification *>(QObject::parent()); }

public: // PROPERTIES
public Q_SLOTS: // METHODS
    void GetReady();
    void NewPrinter(int status, const QString &name, const QString &mfg, const QString &mdl, const QString &des, const QString &cmd);
Q_SIGNALS: // SIGNALS
};

#endif
