/****************************************************************************
** Meta object code from reading C++ file 'KCupsRequest.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../print-manager-6.7.4/src/libkcups/KCupsRequest.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KCupsRequest.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12KCupsRequestE_t {};
} // unnamed namespace

template <> constexpr inline auto KCupsRequest::qt_create_metaobjectdata<qt_meta_tag_ZN12KCupsRequestE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KCupsRequest",
        "device",
        "",
        "device_class",
        "device_id",
        "device_info",
        "device_make_and_model",
        "device_uri",
        "device_location",
        "finished",
        "KCupsRequest*",
        "getPPDS",
        "make",
        "getDevices",
        "timeout",
        "includeSchemes",
        "excludeSchemes",
        "getPrinters",
        "attributes",
        "mask",
        "getPrinterAttributes",
        "printerName",
        "isClass",
        "getJobs",
        "myJobs",
        "whichJobs",
        "getJobAttributes",
        "jobId",
        "printerUri",
        "getServerSettings",
        "getPrinterPPD",
        "setServerSettings",
        "KCupsServer",
        "server",
        "printCommand",
        "command",
        "title",
        "process",
        "KIppRequest",
        "request"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'device'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, const QString &, const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 }, { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
            { QMetaType::QString, 7 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(KCupsRequest *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Method 'getPPDS'
        QtMocHelpers::MethodData<void(const QString &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 12 },
        }}),
        // Method 'getPPDS'
        QtMocHelpers::MethodData<void()>(11, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Method 'getDevices'
        QtMocHelpers::MethodData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Method 'getDevices'
        QtMocHelpers::MethodData<void()>(13, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Method 'getDevices'
        QtMocHelpers::MethodData<void(int, QStringList, QStringList)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 }, { QMetaType::QStringList, 15 }, { QMetaType::QStringList, 16 },
        }}),
        // Method 'getPrinters'
        QtMocHelpers::MethodData<void(QStringList, int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 18 }, { QMetaType::Int, 19 },
        }}),
        // Method 'getPrinters'
        QtMocHelpers::MethodData<void(QStringList)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QStringList, 18 },
        }}),
        // Method 'getPrinterAttributes'
        QtMocHelpers::MethodData<void(const QString &, bool, QStringList)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 }, { QMetaType::Bool, 22 }, { QMetaType::QStringList, 18 },
        }}),
        // Method 'getJobs'
        QtMocHelpers::MethodData<void(const QString &, bool, int, QStringList)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 }, { QMetaType::Bool, 24 }, { QMetaType::Int, 25 }, { QMetaType::QStringList, 18 },
        }}),
        // Method 'getJobAttributes'
        QtMocHelpers::MethodData<void(int, const QString &, QStringList)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 27 }, { QMetaType::QString, 28 }, { QMetaType::QStringList, 18 },
        }}),
        // Method 'getServerSettings'
        QtMocHelpers::MethodData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'getPrinterPPD'
        QtMocHelpers::MethodData<void(const QString &)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Method 'setServerSettings'
        QtMocHelpers::MethodData<void(const KCupsServer &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Method 'printCommand'
        QtMocHelpers::MethodData<void(const QString &, const QString &, const QString &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 }, { QMetaType::QString, 35 }, { QMetaType::QString, 36 },
        }}),
        // Method 'process'
        QtMocHelpers::MethodData<void(const KIppRequest &)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KCupsRequest, qt_meta_tag_ZN12KCupsRequestE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KCupsRequest::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KCupsRequestE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KCupsRequestE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12KCupsRequestE_t>.metaTypes,
    nullptr
} };

void KCupsRequest::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KCupsRequest *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->device((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[6]))); break;
        case 1: _t->finished((*reinterpret_cast<std::add_pointer_t<KCupsRequest*>>(_a[1]))); break;
        case 2: _t->getPPDS((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->getPPDS(); break;
        case 4: _t->getDevices((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->getDevices(); break;
        case 6: _t->getDevices((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[3]))); break;
        case 7: _t->getPrinters((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->getPrinters((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 9: _t->getPrinterAttributes((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[3]))); break;
        case 10: _t->getJobs((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[4]))); break;
        case 11: _t->getJobAttributes((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[3]))); break;
        case 12: _t->getServerSettings(); break;
        case 13: _t->getPrinterPPD((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->setServerSettings((*reinterpret_cast<std::add_pointer_t<KCupsServer>>(_a[1]))); break;
        case 15: _t->printCommand((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 16: _t->process((*reinterpret_cast<std::add_pointer_t<KIppRequest>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KCupsRequest* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KCupsServer >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KIppRequest >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KCupsRequest::*)(const QString & , const QString & , const QString & , const QString & , const QString & , const QString & )>(_a, &KCupsRequest::device, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsRequest::*)(KCupsRequest * )>(_a, &KCupsRequest::finished, 1))
            return;
    }
}

const QMetaObject *KCupsRequest::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KCupsRequest::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KCupsRequestE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KCupsRequest::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void KCupsRequest::device(const QString & _t1, const QString & _t2, const QString & _t3, const QString & _t4, const QString & _t5, const QString & _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}

// SIGNAL 1
void KCupsRequest::finished(KCupsRequest * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
