/****************************************************************************
** Meta object code from reading C++ file 'KCupsConnection.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../print-manager-6.7.4/src/libkcups/KCupsConnection.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'KCupsConnection.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15KCupsConnectionE_t {};
} // unnamed namespace

template <> constexpr inline auto KCupsConnection::qt_create_metaobjectdata<qt_meta_tag_ZN15KCupsConnectionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KCupsConnection",
        "serverStarted",
        "",
        "text",
        "serverStopped",
        "serverRestarted",
        "serverAudit",
        "printerAdded",
        "printerUri",
        "printerName",
        "printerState",
        "printerStateReasons",
        "printerIsAcceptingJobs",
        "printerModified",
        "printerDeleted",
        "printerStateChanged",
        "printerStopped",
        "printerRestarted",
        "printerShutdown",
        "printerMediaChanged",
        "printerFinishingsChanged",
        "jobState",
        "jobId",
        "jobStateReasons",
        "jobName",
        "jobImpressionsCompleted",
        "jobCreated",
        "jobStopped",
        "jobConfigChanged",
        "jobProgress",
        "jobCompleted",
        "updateSubscription",
        "renewDBusSubscription",
        "cancelDBusSubscription"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'serverStarted'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'serverStopped'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'serverRestarted'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'serverAudit'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'printerAdded'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Signal 'printerModified'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Signal 'printerDeleted'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Signal 'printerStateChanged'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Signal 'printerStopped'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Signal 'printerRestarted'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Signal 'printerShutdown'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Signal 'printerMediaChanged'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Signal 'printerFinishingsChanged'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 },
        }}),
        // Signal 'jobState'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, uint, const QString &, bool, uint, uint, const QString &, const QString &, uint)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 }, { QMetaType::UInt, 22 }, { QMetaType::UInt, 21 },
            { QMetaType::QString, 23 }, { QMetaType::QString, 24 }, { QMetaType::UInt, 25 },
        }}),
        // Signal 'jobCreated'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, uint, const QString &, bool, uint, uint, const QString &, const QString &, uint)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 }, { QMetaType::UInt, 22 }, { QMetaType::UInt, 21 },
            { QMetaType::QString, 23 }, { QMetaType::QString, 24 }, { QMetaType::UInt, 25 },
        }}),
        // Signal 'jobStopped'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, uint, const QString &, bool, uint, uint, const QString &, const QString &, uint)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 }, { QMetaType::UInt, 22 }, { QMetaType::UInt, 21 },
            { QMetaType::QString, 23 }, { QMetaType::QString, 24 }, { QMetaType::UInt, 25 },
        }}),
        // Signal 'jobConfigChanged'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, uint, const QString &, bool, uint, uint, const QString &, const QString &, uint)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 }, { QMetaType::UInt, 22 }, { QMetaType::UInt, 21 },
            { QMetaType::QString, 23 }, { QMetaType::QString, 24 }, { QMetaType::UInt, 25 },
        }}),
        // Signal 'jobProgress'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, uint, const QString &, bool, uint, uint, const QString &, const QString &, uint)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 }, { QMetaType::UInt, 22 }, { QMetaType::UInt, 21 },
            { QMetaType::QString, 23 }, { QMetaType::QString, 24 }, { QMetaType::UInt, 25 },
        }}),
        // Signal 'jobCompleted'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, uint, const QString &, bool, uint, uint, const QString &, const QString &, uint)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 }, { QMetaType::UInt, 10 },
            { QMetaType::QString, 11 }, { QMetaType::Bool, 12 }, { QMetaType::UInt, 22 }, { QMetaType::UInt, 21 },
            { QMetaType::QString, 23 }, { QMetaType::QString, 24 }, { QMetaType::UInt, 25 },
        }}),
        // Slot 'updateSubscription'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'renewDBusSubscription'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'cancelDBusSubscription'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KCupsConnection, qt_meta_tag_ZN15KCupsConnectionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KCupsConnection::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15KCupsConnectionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15KCupsConnectionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15KCupsConnectionE_t>.metaTypes,
    nullptr
} };

void KCupsConnection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KCupsConnection *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->serverStarted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->serverStopped((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->serverRestarted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->serverAudit((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->printerAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 5: _t->printerModified((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 6: _t->printerDeleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 7: _t->printerStateChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 8: _t->printerStopped((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 9: _t->printerRestarted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 10: _t->printerShutdown((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 11: _t->printerMediaChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 12: _t->printerFinishingsChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 13: _t->jobState((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[11]))); break;
        case 14: _t->jobCreated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[11]))); break;
        case 15: _t->jobStopped((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[11]))); break;
        case 16: _t->jobConfigChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[11]))); break;
        case 17: _t->jobProgress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[11]))); break;
        case 18: _t->jobCompleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[11]))); break;
        case 19: _t->updateSubscription(); break;
        case 20: _t->renewDBusSubscription(); break;
        case 21: _t->cancelDBusSubscription(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & )>(_a, &KCupsConnection::serverStarted, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & )>(_a, &KCupsConnection::serverStopped, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & )>(_a, &KCupsConnection::serverRestarted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & )>(_a, &KCupsConnection::serverAudit, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & , const QString & , const QString & , uint , const QString & , bool )>(_a, &KCupsConnection::printerAdded, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & , const QString & , const QString & , uint , const QString & , bool )>(_a, &KCupsConnection::printerModified, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & , const QString & , const QString & , uint , const QString & , bool )>(_a, &KCupsConnection::printerDeleted, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & , const QString & , const QString & , uint , const QString & , bool )>(_a, &KCupsConnection::printerStateChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & , const QString & , const QString & , uint , const QString & , bool )>(_a, &KCupsConnection::printerStopped, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & , const QString & , const QString & , uint , const QString & , bool )>(_a, &KCupsConnection::printerRestarted, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & , const QString & , const QString & , uint , const QString & , bool )>(_a, &KCupsConnection::printerShutdown, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & , const QString & , const QString & , uint , const QString & , bool )>(_a, &KCupsConnection::printerMediaChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & , const QString & , const QString & , uint , const QString & , bool )>(_a, &KCupsConnection::printerFinishingsChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & , const QString & , const QString & , uint , const QString & , bool , uint , uint , const QString & , const QString & , uint )>(_a, &KCupsConnection::jobState, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & , const QString & , const QString & , uint , const QString & , bool , uint , uint , const QString & , const QString & , uint )>(_a, &KCupsConnection::jobCreated, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & , const QString & , const QString & , uint , const QString & , bool , uint , uint , const QString & , const QString & , uint )>(_a, &KCupsConnection::jobStopped, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & , const QString & , const QString & , uint , const QString & , bool , uint , uint , const QString & , const QString & , uint )>(_a, &KCupsConnection::jobConfigChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & , const QString & , const QString & , uint , const QString & , bool , uint , uint , const QString & , const QString & , uint )>(_a, &KCupsConnection::jobProgress, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCupsConnection::*)(const QString & , const QString & , const QString & , uint , const QString & , bool , uint , uint , const QString & , const QString & , uint )>(_a, &KCupsConnection::jobCompleted, 18))
            return;
    }
}

const QMetaObject *KCupsConnection::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KCupsConnection::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15KCupsConnectionE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int KCupsConnection::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void KCupsConnection::serverStarted(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KCupsConnection::serverStopped(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KCupsConnection::serverRestarted(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KCupsConnection::serverAudit(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KCupsConnection::printerAdded(const QString & _t1, const QString & _t2, const QString & _t3, uint _t4, const QString & _t5, bool _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}

// SIGNAL 5
void KCupsConnection::printerModified(const QString & _t1, const QString & _t2, const QString & _t3, uint _t4, const QString & _t5, bool _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}

// SIGNAL 6
void KCupsConnection::printerDeleted(const QString & _t1, const QString & _t2, const QString & _t3, uint _t4, const QString & _t5, bool _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}

// SIGNAL 7
void KCupsConnection::printerStateChanged(const QString & _t1, const QString & _t2, const QString & _t3, uint _t4, const QString & _t5, bool _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}

// SIGNAL 8
void KCupsConnection::printerStopped(const QString & _t1, const QString & _t2, const QString & _t3, uint _t4, const QString & _t5, bool _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}

// SIGNAL 9
void KCupsConnection::printerRestarted(const QString & _t1, const QString & _t2, const QString & _t3, uint _t4, const QString & _t5, bool _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}

// SIGNAL 10
void KCupsConnection::printerShutdown(const QString & _t1, const QString & _t2, const QString & _t3, uint _t4, const QString & _t5, bool _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}

// SIGNAL 11
void KCupsConnection::printerMediaChanged(const QString & _t1, const QString & _t2, const QString & _t3, uint _t4, const QString & _t5, bool _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}

// SIGNAL 12
void KCupsConnection::printerFinishingsChanged(const QString & _t1, const QString & _t2, const QString & _t3, uint _t4, const QString & _t5, bool _t6)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2, _t3, _t4, _t5, _t6);
}

// SIGNAL 13
void KCupsConnection::jobState(const QString & _t1, const QString & _t2, const QString & _t3, uint _t4, const QString & _t5, bool _t6, uint _t7, uint _t8, const QString & _t9, const QString & _t10, uint _t11)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7, _t8, _t9, _t10, _t11);
}

// SIGNAL 14
void KCupsConnection::jobCreated(const QString & _t1, const QString & _t2, const QString & _t3, uint _t4, const QString & _t5, bool _t6, uint _t7, uint _t8, const QString & _t9, const QString & _t10, uint _t11)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7, _t8, _t9, _t10, _t11);
}

// SIGNAL 15
void KCupsConnection::jobStopped(const QString & _t1, const QString & _t2, const QString & _t3, uint _t4, const QString & _t5, bool _t6, uint _t7, uint _t8, const QString & _t9, const QString & _t10, uint _t11)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7, _t8, _t9, _t10, _t11);
}

// SIGNAL 16
void KCupsConnection::jobConfigChanged(const QString & _t1, const QString & _t2, const QString & _t3, uint _t4, const QString & _t5, bool _t6, uint _t7, uint _t8, const QString & _t9, const QString & _t10, uint _t11)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7, _t8, _t9, _t10, _t11);
}

// SIGNAL 17
void KCupsConnection::jobProgress(const QString & _t1, const QString & _t2, const QString & _t3, uint _t4, const QString & _t5, bool _t6, uint _t7, uint _t8, const QString & _t9, const QString & _t10, uint _t11)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7, _t8, _t9, _t10, _t11);
}

// SIGNAL 18
void KCupsConnection::jobCompleted(const QString & _t1, const QString & _t2, const QString & _t3, uint _t4, const QString & _t5, bool _t6, uint _t7, uint _t8, const QString & _t9, const QString & _t10, uint _t11)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1, _t2, _t3, _t4, _t5, _t6, _t7, _t8, _t9, _t10, _t11);
}
QT_WARNING_POP
